package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.shipment.convert.makeBillOfLading.MakeBillOfLadingConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.makeBillOfLading.MakeBillOfLadingDO;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.makeBillOfLading.MakeBillOfLadingService;
import cn.iocoder.yudao.module.shipment.vo.makeBillOfLading.MakeBillOfLadingBackVO;
import cn.iocoder.yudao.module.shipment.vo.makeBillOfLading.MakeBillOfLadingCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.makeBillOfLading.MakeBillOfLadingQueryVO;
import cn.iocoder.yudao.module.shipment.vo.makeBillOfLading.MakeBillOfLadingUpdateReqVO;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;

@Validated
@RestController
@Api(tags = "管理后台 - 制作提货单")
@RequestMapping("/shipment/make-bill-of-lading")
@Slf4j
public class MakeBillOfLadingController {

    @Value("${ue.templates-url}")
    private String path;

    @Resource
    private MakeBillOfLadingService makeBillOfLadingService;

    @Resource
    private OrderService orderService;

    @Resource
    private BoxService boxService;

    @Resource
    private FileMakeApi fileMakeApi;

    @Resource
    private RedisHelper redisHelper;

    @Value("${ue.root-path}")
    private String prePath;

    private static final String BOX_MAKE_LADING_KEY = "jiedao:box:make:lading:{0}";

    @PostMapping("/create")
    @ApiOperation("创建制作提货单")
    @Idempotent(timeout = 10)
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:create')")
    public CommonResult<Long> createMakeBillOfLading(@Valid @RequestBody MakeBillOfLadingCreateReqVO createReqVO) {
        //修改此函数，取消审批流程
        //取消流程的发起
        //ecw_make_bill_of_lading 提单
        String redisKey = MessageFormat.format(BOX_MAKE_LADING_KEY, createReqVO.getOrderId().toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(BOX_MAKE_LADING_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        Long makeBillOfLading =
                makeBillOfLadingService.createMakeBillOfLading(createReqVO);
        redisHelper.delete(redisKey);
        return success(makeBillOfLading);
    }

    @PutMapping("/update")
    @ApiOperation("更新制作提货单")
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:update')")
    public CommonResult<Boolean> updateMakeBillOfLading(
            @Valid @RequestBody MakeBillOfLadingUpdateReqVO updateReqVO) {
        makeBillOfLadingService.updateMakeBillOfLading(updateReqVO);
        return success(true);
    }

    @PutMapping("/updateUrl")
    @ApiOperation("更新提货单url")
    public CommonResult<Boolean> updateMakeBillOfLadingUrl(@Valid @RequestBody MakeBillOfLadingUpdateReqVO updateReqVO) {
        makeBillOfLadingService.updateMakeBillOfLadingUrl(updateReqVO);
        return success(true);
    }

    @GetMapping("/updateAirBill")
    @ApiOperation("刷新空运提单")
    @ApiImplicitParam(name = "ids", value = "订单ID列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> updateAirBill(@RequestParam("ids") Collection<Long> orderIds) {
        makeBillOfLadingService.updateAirBill(orderIds);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除制作提货单")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:delete')")
    public CommonResult<Boolean> deleteMakeBillOfLading(@RequestParam("id") Long id) {
        makeBillOfLadingService.deleteMakeBillOfLading(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得制作提货单")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
    public CommonResult<MakeBillOfLadingBackVO> getMakeBillOfLading(@RequestParam("id") Long id) {
        MakeBillOfLadingDO makeBillOfLading = makeBillOfLadingService.getMakeBillOfLading(id);
        return success(MakeBillOfLadingConvert.INSTANCE.convert(makeBillOfLading));
    }

    @GetMapping("/getInProcessing")
    @ApiOperation("获得制作提货单(审核中调用)")
    @ApiImplicitParam(name = "orderId", value = "订单编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
    public CommonResult<MakeBillOfLadingBackVO> getInProcessing(@RequestParam("orderId") Long orderId) {
        MakeBillOfLadingDO makeBillOfLading = makeBillOfLadingService.getInProcessing(orderId);
        return success(MakeBillOfLadingConvert.INSTANCE.convert(makeBillOfLading));
    }

    @GetMapping("/zipDownload")
    @ApiOperation("打包下载制作提货单")
    @ApiImplicitParam(name = "shipmentId", value = "出货编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
    public CommonResult<String> zipDownload(@RequestParam("shipmentId") Long shipmentId, HttpServletResponse response) throws Exception {
        Long aLong = makeBillOfLadingService.selectCount(new LambdaQueryWrapperX<MakeBillOfLadingDO>().eq(MakeBillOfLadingDO::getShipmentId, shipmentId).eq(MakeBillOfLadingDO::getStatus, 2));
        if (aLong == null || aLong <= 0L) {
            return error(HAVE_NOT_LADING);
        }
        BoxDO box = boxService.getBox(shipmentId);
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("shipmentId", shipmentId);
        reqDTO.setType(DownloadTypeEnum.SHIPMENT_BILLING_ZIP_EXPORT.getType());
        reqDTO.setName(box.getSelfNo() + "提单导出Zip");
        reqDTO.setFileSuffix("zip");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(jsonObject.toJSONString());
        fileMakeApi.sendFileMake(reqDTO);
        return success("");
    }

    @GetMapping("/downloadByOrderId")
    @ApiOperation("下载制作提货单（订单ID）")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
    public CommonResult<MakeBillOfLadingBackVO> downloadByOrderId(@RequestParam("orderId") Long orderId, HttpServletResponse response) {
        MakeBillOfLadingDO lading =
                makeBillOfLadingService.selectOne(MakeBillOfLadingDO::getOrderId,
                        orderId);
        if (lading == null) {
            return error(new ErrorCode(1109010018,
                    "该提单未制作或未审批通过！"));
        }
        OrderDO orderDOCheck = orderService.getById(orderId);
        if (orderDOCheck.getStatus() < OrderStatusEnum.INSTALLED_CABINET.getValue()) {
            return error(new ErrorCode(1109010019, "装柜前无法下载提货单"));
        }
        MakeBillOfLadingBackVO vResult =
                MakeBillOfLadingConvert.INSTANCE.convert(lading);

        //前端是根据 status 来判断的，此处逻辑可以不用理会
        if (lading.getBpmProcessId() == null ||
                lading.getBpmProcessId().length() == 0) {
            vResult.setBpmStatus("-1");
        } else {
            String sStatus =
                    makeBillOfLadingService.getBpmStatus(lading.getBpmProcessId());
            if (sStatus == null) sStatus = "-1";
            vResult.setBpmStatus(sStatus);
        }
        return success(vResult);
    }

    @GetMapping("/download")
    @ApiOperation("下载制作提货单")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
    public CommonResult<MakeBillOfLadingBackVO> download(@RequestParam("id") Long id, HttpServletResponse response) {
        MakeBillOfLadingDO makeBillOfLading =
                makeBillOfLadingService.getMakeBillOfLading(id);
        return success(MakeBillOfLadingConvert.INSTANCE.convert(makeBillOfLading));
    }

    @GetMapping("/getMakeLadingBillList")
    @ApiOperation("获得制作提货单列表 use this")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult<Map<String, Object>> getMakeLadingBillList(@RequestParam("shipmentId") Long shipmentId) {
        Map<String, Object> map = makeBillOfLadingService.getMakeLadingBillList(shipmentId);
        return success(map);
    }

    @DeleteMapping("/cancel")
    @ApiOperation("取消制作提货单审核")
    @Idempotent(timeout = 5)
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:delete')")
    public CommonResult<Boolean> cancel(@RequestParam("id") Long id) {
        makeBillOfLadingService.cancel(id);
        return success(true);
    }

    /*
     修改报关价显示单位问题
     */
    @GetMapping("/make")
    @ApiOperation("制作提货单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "订单编号", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "shipmentId", value = "出货ID", required = true, example = "1024", dataTypeClass = Long.class)
    })
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
    public CommonResult<Map<String, Object>> makeBillOfLading(@RequestParam("orderId") Long orderId, @RequestParam("shipmentId") Long shipmentId) throws Exception {
        Map<String, Object> map = makeBillOfLadingService.makeBillOfLading(orderId, shipmentId);
        return success(map);
    }

    @GetMapping("/airMake")
    @ApiOperation("空运制作提货单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "订单编号", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "shipmentId", value = "出货ID", required = true, example = "1024", dataTypeClass = Long.class)
    })
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
    public CommonResult<Map<String, Object>> makeAirBillOfLading(@RequestParam("orderId") Long orderId, @RequestParam("shipmentId") Long shipmentId) throws Exception {
        Map<String, Object> map =
                makeBillOfLadingService.makeAirBillOfLading(orderId, shipmentId);
        return success(map);
    }

    @GetMapping("/list")
    @ApiOperation("获得制作提货单列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
    public CommonResult<List<MakeBillOfLadingBackVO>> getMakeBillOfLadingList(@RequestParam("ids") Collection<Long> ids) {
        List<MakeBillOfLadingDO> list = makeBillOfLadingService.getMakeBillOfLadingList(ids);
        return success(MakeBillOfLadingConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得制作提货单分页")
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
    public CommonResult<PageResult<MakeBillOfLadingBackVO>> getMakeBillOfLadingPage(@Valid MakeBillOfLadingQueryVO query, PageVO page) {
        PageResult<MakeBillOfLadingDO> pageResult = makeBillOfLadingService.getMakeBillOfLadingPage(query, page);
        return success(MakeBillOfLadingConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出制作提货单 Excel")
    //@PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:export')")
    @OperateLog(type = EXPORT)
    public void exportMakeBillOfLadingExcel(@Valid MakeBillOfLadingQueryVO query,
                                            HttpServletResponse response) throws IOException {
        List<MakeBillOfLadingDO> list = makeBillOfLadingService.getMakeBillOfLadingList(query);
        // 导出 Excel
        List<MakeBillOfLadingBackVO> datas = MakeBillOfLadingConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "制作提货单.xls", "数据", MakeBillOfLadingBackVO.class, datas);
    }
}
